#!/usr/bin/env node
(function() {
  var Worker, app, cluster;

  cluster = require('cluster');

  app = process.argv[2];

  Worker = (function() {
    var getWorkerIndex, workers;

    workers = [];

    getWorkerIndex = function(proc) {
      var k, v, _len;
      for (k = 0, _len = workers.length; k < _len; k++) {
        v = workers[k];
        if (v.proc === proc) return k;
      }
    };

    cluster.on('death', function(proc) {
      var i;
      i = getWorkerIndex(proc);
      if (proc.exitCode === 0) {
        return workers.splice(i, 1);
      } else {
        return workers[i].fork();
      }
    });

    function Worker(msg) {
      this.msg = msg != null ? msg : {};
      workers.push(this);
      this.fork();
    }

    Worker.prototype.fork = function() {
      var proc;
      var _this = this;
      process.env.APP = app;
      this.proc = proc = cluster.fork();
      return proc.on('message', function(msg) {
        var worker, _i, _len, _results;
        if (msg.cmd === 'zion:online') {
          return proc.send(_this.msg);
        } else if (msg.cmd === 'zion:fork') {
          return new Worker(msg);
        } else if (msg.cmd === 'zion:restart') {
          _results = [];
          for (_i = 0, _len = workers.length; _i < _len; _i++) {
            worker = workers[_i];
            _results.push(process.kill(worker.proc.pid));
          }
          return _results;
        }
      });
    };

    return Worker;

  })();

  if (cluster.isMaster) {
    new Worker({
      cmd: 'zion:bootstrap'
    });
  } else {
    process.send({
      cmd: 'zion:online'
    });
    process.on('message', function(msg) {
      return require(process.env.APP)(msg);
    });
  }

}).call(this);
